/**************************************************************
***** Creates APPX Connector service process for Windows, *****
************** Linux, and Mac operating systems ***************
**************************************************************/

var serverConnectorInstallVersionStr = "5.4.3";
var serverConnectorInstallVersionNum = 50403.014;

var exec = require('child_process').execSync, child;

// Make sure the requird modules are installed.
var modules = [
    'crypto',
    'fs',
    'hexy',
    'http',
    'https',
    'mongodb',
    'net',
    'node-cryptojs-aes',
    'string',
    'string_decoder',
    'url',
    'ws'
    ];

console.log("");
console.log("Checking for required modules...");

var allReqInstalled = true;
modules.forEach(function(name){
	if( ! testForModule(name) )
	    allReqInstalled = false;
});

console.log("");

if( ! allReqInstalled ) {
    console.log("Missing modules.  Please install required modules and run again.");
    process.exit();
}

//Detects OS and sets the Service variable accordingly.
switch(process.platform){
	case 'win32':
		var Service = require('node-windows').Service; break;
	case 'linux':
		var Service = require('node-linux').Service; break;
	case 'darwin':
		var Service = require('node-mac').Service; break;
}


console.log("Installing appxconnector service...");

//Create new service object
var svc = new Service({
	name: 'appxConnector',
	description: 'Connector that allows appx client to talk to server.',
	script: 'appxConnector.js',
	
	//Set variable so appxConnector.js knows where to push files to.
	env:[{
		name: "APPX_CONNECTOR_DIR",
		value: require('path').resolve(__dirname)
	},{
		name: "NODE_ENV",
		value: "production"
	}]
});
	
//Installs service
svc.on('alreadyinstalled', function(){
	console.log("Already installed, restarting appxconnector service...");
	svc.stop();
	svc.start();
	console.log("Done");
    });

svc.on('install', function(){
	console.log("Installing appxconnector service...");
	setTimeout(function(){
	svc.start();
	},100);
	
	console.log("Done");
    });

	svc.install();

function testForModule(name) {
    var nameStr = name + "                              ";
    process.stdout.write("Module: " + nameStr.substring(0,25) + " - ");
    if( isModuleInstalled(name) ) {
	console.log("Installed.");
	return true;
    }
    else {
	console.log("Missing, please run 'npm install " + name + "' to install.");
	return false;
    }
}

function isModuleInstalled(name) {
    try {
	require.resolve(name);
	return true;
    }
    catch(ex) {
	return false;
    }
}
